package net.wanji.web.service.scheme.impl;

import net.wanji.web.dto.SaveSchemeConfigDTO;
import net.wanji.web.mapper.scheme.CrossPhaseMapper;
import net.wanji.web.mapper.scheme.CrossSchemeMapper;
import net.wanji.web.po.scheme.CrossSchemePO;
import net.wanji.web.service.scheme.SchemeConfigService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author Kent HAN
 * @date 2023/1/5 10:27
 */
@Service
public class SchemeConfigServiceImpl implements SchemeConfigService {
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;

    public SchemeConfigServiceImpl(CrossSchemeMapper crossSchemeMapper, CrossPhaseMapper crossPhaseMapper) {
        this.crossSchemeMapper = crossSchemeMapper;
        this.crossPhaseMapper = crossPhaseMapper;
    }

    @Override
    @Transactional
    public void saveSchemeConfig(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        Integer timeType = saveSchemeConfigDTO.getTimeType();
        if (timeType == 2) { //环式
            handleRingTime(saveSchemeConfigDTO);
        } else if (timeType == 1) { //阶段式
            // TODO: 2023/1/6 处理阶段式
        }
    }

    private void handleRingTime(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        // 更新方案基础信息表
        updateCrossScheme(saveSchemeConfigDTO);
        // 更新相位基础表
    }

    private void updateCrossScheme(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        crossSchemeMapper.deleteByCrossId(crossId);
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        List<CrossSchemePO> crossSchemePOList = new ArrayList<>();
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            CrossSchemePO crossSchemePO = new CrossSchemePO();
            String schemeNo = phaseScheme.getSchemeNo();// 方案号
            crossSchemePO.setSchemeNo(schemeNo);
            String name = phaseScheme.getName(); // 方案名称
            crossSchemePO.setName(name);
            crossSchemePO.setCrossId(crossId);
            // 获取方案周期、协调相位ID、相位差
            List<SaveSchemeConfigDTO.TimeScheme> timeSchemeList = saveSchemeConfigDTO.getTimeSchemeList();
            for (SaveSchemeConfigDTO.TimeScheme timeScheme : timeSchemeList) {
                String schemeNoInTime = timeScheme.getSchemeNo();
                if (Objects.equals(schemeNo, schemeNoInTime)) {
                    crossSchemePO.setCycle(timeScheme.getCycle());
                    String coordPhaseNo = timeScheme.getCoordPhaseNo();
                    Integer coordPhaseId = crossPhaseMapper.selectIdByCrossIdAndPhaseNo(crossId, coordPhaseNo);
                    crossSchemePO.setCoordPhase(coordPhaseId);
                    crossSchemePO.setOffset(timeScheme.getOffset());
                }
            }
            crossSchemePO.setSource(2); // 来自平台
            crossSchemePO.setIsDeleted(0);
            crossSchemePOList.add(crossSchemePO);
        }
        crossSchemeMapper.insertBatch(crossSchemePOList);
    }
}
